{% load i18n staticfiles %}
<div class="inline-group">
	<div class="tabular inline-related {% if forloop.last %}last-related{% endif %}">
		{{ inline_admin_formset.formset.management_form }}
		<fieldset class="module">
			<h2>{{ inline_admin_formset.opts.verbose_name_plural|capfirst }}</h2>
			{{ inline_admin_formset.formset.non_form_errors }}
			<table>
				<thead><tr>
					{% for field in inline_admin_formset.fields %}
					<th {% if forloop.first %}colspan="2"{% endif %}>{{ field.label|capfirst }}</th>
					{% endfor %}
					{% if inline_admin_formset.formset.can_delete %}<th>{% trans "Delete?" %}</th>{% endif %}
				</tr></thead>
				<tbody>
					{% for inline_admin_form in inline_admin_formset %}
					{% if inline_admin_form.form.non_field_errors %}
						<tr><td colspan="{{ inline_admin_form.field_count }}">{{ inline_admin_form.form.non_field_errors }}</td></tr>
					{% endif %}
					<tr class="{% cycle row1,row2 %} {% if inline_admin_form.original or inline_admin_form.show_url %}has_original{% endif %}">
						<td class="original">
							{% if inline_admin_form.original or inline_admin_form.show_url %}<p>
								{% if inline_admin_form.original %} {{ inline_admin_form.original.content_object }}{% endif %}
								{% if inline_admin_form.show_url %}<a href="../../../r/{{ inline_admin_form.original.content_type_id }}/{{ inline_admin_form.original.id }}/">{% trans "View on site" %}</a>{% endif %}
							</p>{% endif %}
							{% if inline_admin_form.has_auto_field %}{{ inline_admin_form.pk_field.field }}{% endif %}
							{{ inline_admin_form.fk_field.field }}
							{% spaceless %}
							{% for fieldset in inline_admin_form %}
								{% for line in fieldset %}
									{% for field in line %}
									{% if field.is_hidden %} {{ field.field }} {% endif %}
									{% endfor %}
								{% endfor %}
							{% endfor %}
							{% endspaceless %}
						</td>
						{% for fieldset in inline_admin_form %}
							{% for line in fieldset %}
								{% for field in line %}
									<td class="{{ field.field.name }}">
										{% if field.is_readonly %}
											<p>{{ field.contents }}</p>
										{% else %}
											{{ field.field.errors.as_ul }}
											{% ifequal field.field.name inline_admin_formset.formset.ct_fk_field %}
												{{ field.field }} 
												<a id="lookup_id_{{field.field.html_name}}" class="related-lookup" onclick="return showGenericRelatedObjectLookupPopup(this, {{ inline_admin_formset.formset.content_types }});" href="#">
													<img width="16" height="16" alt="Lookup" src="{% static 'img/admin/selector-search.gif' %}"/>
												</a>
											{% else %}{{ field.field }} {% endifequal %}
										{% endif %}
									</td>
								{% endfor %}
							{% endfor %}
						{% endfor %}
						{% if inline_admin_formset.formset.can_delete %}
						<td class="delete">
							{% if inline_admin_form.original %}{{ inline_admin_form.deletion_field.field }}{% endif %}
						</td>
						{% endif %}
					</tr>
					{% endfor %}
				</tbody>
			</table>
		</fieldset>
	</div>
</div>
